import random
import time
from typing import List, Union, overload, Dict

import requests

from pygtrans.DetectResponse import DetectResponse
from pygtrans.Null import Null
from pygtrans.TranslateResponse import TranslateResponse


class Translate:
    """
    :param target: str: (可选) 目标语言, 默认: ``zh-CN``, :doc:`查看完整列表 <target>`
    :param source: str: (可选) 源语言, 默认: ``auto`` (自动检测), :doc:`查看完整列表 <source>`
    :param fmt: str: (可选) 文本格式, ``text`` | ``html``, 默认: ``html``
    :param user_agent: str: (可选) 用户代理, 这个参数很重要, 不设置或错误设置非常容易触发 **429 Too Many Requests** 错误,
        默认: ``GoogleTranslate/6.18.0.06.376053713 (Linux; U; Android 11; GM1900)``, 所以用户可以不用提供.
        这个默认 ``User-Agent`` 很稳定, 暂时未发现 ``429 错误``, 如果出现 ``429``, 建议 **模仿默认 进行构造**,
        或者进行 `反馈 <https://github.com/foyoux/pygtrans/issues/new>`_
    :param domain: str: (可选) 域名 ``google.com`` 及其可用平行域名 (如: ``google.cn``), 默认: ``google.com``
    :param proxies: (可选) eg: proxies = {'http': 'http://localhost:10809', 'https': 'http://localhost:10809'}

    基本用法:
        >>> from pygtrans import Translate
        >>> client = Translate(proxies={'http': 'http://localhost:10809', 'https': 'http://localhost:10809'})
        >>> client.detect('谷歌翻译').language
        'zh-CN'
        >>> text = client.translate('Hello, Google')
        >>> text.translatedText
        '你好，谷歌'
        >>> texts = client.translate(['批量测试', '批量翻译'], target='en')
        >>> for text in texts:
        ...     print(text.translatedText)
        Batch test
        Batch translation
    """

    def __init__(
        self,
        target: str = "zh-CN",
        source: str = "auto",
        fmt="html",
        user_agent: str = None,
        domain: str = "com",
        proxies: Dict = None,
        timeout: int = None,
        trust_env=False,
    ):
        self.target = target
        self.source = source
        self.fmt = fmt
        self.timeout = timeout

        if user_agent is None:
            user_agent = (
                f"GoogleTranslate/6.{random.randint(10, 100)}.0.06.{random.randint(111111111, 999999999)}"
                " (Linux; U; Android {random.randint(5, 11)}; {base64.b64encode(str(random.random())["
                "2:].encode()).decode()}) "
            )

        self.session = requests.Session()
        self.session.headers = {"User-Agent": user_agent}
        self.BASE_URL: str = "https://translate.google." + domain
        self.LANGUAGE_URL: str = f"{self.BASE_URL}/translate_a/l"
        self.DETECT_URL: str = f"{self.BASE_URL}/translate_a/single"
        self.TRANSLATE_URL: str = f"{self.BASE_URL}/translate_a/t"
        self.TTS_URL: str = f"{self.BASE_URL}/translate_tts"
        self.session.trust_env = trust_env

        if proxies:
            self.session.proxies = proxies

    def detect(self, q: str, timeout=...) -> Union[DetectResponse, Null]:
        """语言检测

        :param q: 需要检测的内容, 不支持批量, 如需批量, 请参阅: :func:`translate_and_detect`.
        :param timeout: 超时时间， int | None
        :return: 成功则返回 :class:`pygtrans.DetectResponse.DetectResponse` 对象,
            失败则返回 :class:`pygtrans.Null.Null` 对象

        基本用法:
            >>> from pygtrans import Translate
            >>> client = Translate(proxies={'http': 'http://localhost:10809', 'https': 'http://localhost:10809'})
            >>> d = client.detect('こんにちは')
            >>> assert d.language == 'ja'
        """
        if timeout is ...:
            timeout = self.timeout
        rt = self.translate(q, target="en", timeout=timeout)
        return DetectResponse(language=rt.detectedSourceLanguage)

    @overload
    def translate(
        self,
        q: str,
        target: str = None,
        source: str = None,
        fmt: str = None,
        timeout=...,
    ) -> TranslateResponse:
        """..."""

    @overload
    def translate(
        self,
        q: List[str],
        target: str = None,
        source: str = None,
        fmt: str = None,
        timeout=...,
    ) -> List[TranslateResponse]:
        """..."""

    def translate(
        self,
        q: Union[str, List[str]],
        target: str = None,
        source: str = None,
        fmt: str = None,
        timeout=...,
    ) -> Union[TranslateResponse, List[TranslateResponse], Null]:
        """翻译文本, 支持批量, 支持 html

        :param q: str: 字符串或字符串列表
        :param target: str: (可选)  目标语言, 默认: ``self.target``, :doc:`查看支持列表 <target>`
        :param source: str: (可选)  源语言, 默认: ``self.source``, :doc:`查看支持列表 <source>`
        :param fmt: str: (可选) 文本格式, ``text`` | ``html``, 默认: ``self.format``
        :param timeout: 超时时间， int | None
        :return: 成功则返回: :class:`pygtrans.TranslateResponse.TranslateResponse` 对象,
            或 :class:`pygtrans.TranslateResponse.TranslateResponse` 对象列表, 这取决于 `参数: q` 是字符串还是字符串列表.
            失败则返回 :class:`pygtrans.Null.Null` 对象

        基本用法:
            >>> from pygtrans import Translate
            >>> client = Translate(proxies={'http': 'http://localhost:10809', 'https': 'http://localhost:10809'})
            >>> text = client.translate('Hello, Google')
            >>> text.translatedText
            '你好，谷歌'
            >>> texts = client.translate(['批量测试', '批量翻译'], target='en')
            >>> for text in texts:
            ...     print(text.translatedText)
            Batch test
            Batch translation
        """

        if not q:
            return []

        if timeout is ...:
            timeout = self.timeout

        if isinstance(q, str):
            if q == "":
                return TranslateResponse("")

        for i in range(1, 4):
            response = self.__translate(
                q=q, target=target, source=source, fmt=fmt, v="1.0", timeout=timeout
            )
            if response.status_code == 429:
                time.sleep(5 * i)
                continue
            break
        # noinspection PyUnboundLocalVariable
        if response.status_code == 200:
            ll = [TranslateResponse(translatedText=i) for i in response.json()]
            if isinstance(q, str):
                return ll[0]
            return ll

        return Null(response)

    def __translate(
        self,
        q: Union[str, List[str]],
        target: str = None,
        source: str = None,
        fmt: str = None,
        v: str = None,
        timeout=...,
    ):
        if target is None:
            target = self.target
        if source is None:
            source = self.source
        if fmt is None:
            fmt = self.fmt
        if timeout is ...:
            timeout = self.timeout
        for i in range(1, 4):
            response = self.session.post(
                self.TRANSLATE_URL,
                params={
                    "tl": target,
                    "sl": source,
                    "ie": "UTF-8",
                    "oe": "UTF-8",
                    "client": "at",
                    "dj": "1",
                    "format": fmt,
                    "v": v,
                },
                data={"q": q},
                timeout=timeout,
            )
            if response.status_code == 429:
                time.sleep(5 * i)
                continue
            break
        # noinspection PyUnboundLocalVariable
        return response

    def tts(self, q: str, target: str = None, timeout=...) -> Union[bytes, Null]:
        """语音: 实验性功能

        :param q: 只支持短语字符串
        :param target: 目标语言
        :param timeout: 超时时间， int | None
        :return: 返回二进制数据, 需要自行写入文件, MP3
        """
        if target is None:
            target = self.target

        if timeout is ...:
            timeout = self.timeout

        for i in range(1, 4):
            response = self.session.get(
                self.TTS_URL,
                params={"ie": "UTF-8", "client": "at", "tl": target, "q": q},
                timeout=timeout,
            )
            if response.status_code == 429:
                time.sleep(5 * i)
                continue
            break
        # noinspection PyUnboundLocalVariable
        if response.status_code == 200:
            return response.content
        return Null(response)



import math
import time
from typing import List, Union, Dict, overload

import requests

from pygtrans.DetectResponse import DetectResponse
from pygtrans.LanguageResponse import LanguageResponse
from pygtrans.Null import Null
from pygtrans.TranslateResponse import TranslateResponse


def split_list(obj_list: List, sub_size: int = 128) -> List[list]:
    if not isinstance(obj_list, list):
        return [[obj_list]]
    if sub_size < 1:
        sub_size = 1
    return [obj_list[i : i + sub_size] for i in range(0, len(obj_list), sub_size)]


def split_list_by_content_size(
    obj_list: List[str], content_size: int = 102400
) -> List[List[str]]:
    """..."""
    if content_size < 1:
        content_size = 1
    if len(obj_list) == 1 or len("".join(obj_list)) <= content_size:
        return [obj_list]

    mid = math.ceil(len(obj_list) / 2)
    ll = []
    ll.extend(split_list_by_content_size(obj_list[:mid], content_size=content_size))
    ll.extend(split_list_by_content_size(obj_list[mid:], content_size=content_size))
    return ll


class ApiKeyTranslate:
    """
    :param api_key: str: 谷歌云翻译APIKEY, `查看详情 <https://cloud.google.com/docs/authentication/api-keys>`_
    :param target: str: (可选) 目标语言, 默认: ``zh-CN``, :doc:`参考列表 <target>`
    :param source: str: (可选) 源语言, 默认: ``auto`` (自动检测), :doc:`参考列表 <source>`
    :param fmt: str: (可选) 文本格式, ``text`` | ``html``, 默认: ``html``
    :param model: str: (可选) 翻译模型. 可以是 base 使用 Phrase-Based Machine Translation (PBMT) 模型，
        或者 nmt 使用 Neural Machine Translation (NMT) 模型。如果省略，则使用 nmt。如果模型是 nmt，
        并且 NMT 模型不支持请求的语言翻译对，则使用 PBMT 模型翻译请求。
    :param proxies: (可选) eg: `proxies = {'http': 'http://localhost:10809','https': 'http://localhost:10809'}`

    基本用法:
        >>> from pygtrans import ApiKeyTranslate
        >>> client = ApiKeyTranslate(api_key='<api_key>')
        >>> langs = client.languages()  # 此种方式的语言列表, 请使用此方法获取
        >>> langs[0]
        LanguageResponse(language='sq', name='阿尔巴尼亚语')
        >>> text = client.translate('Google Translate')
        >>> text.translatedText
        '谷歌翻译'
        >>> text.detectedSourceLanguage
        'en'
        >>> texts = client.translate(['안녕하십니까', 'こんにちは'])
        >>> texts[0].translatedText
        '你好'
        >>> texts[0].detectedSourceLanguage
        'ko'
        >>> texts[1].translatedText
        '你好'
        >>> texts[1].detectedSourceLanguage
        'ja'
    """

    _BASE_URL: str = "https://translation.googleapis.com/language/translate/v2"
    _LANGUAGE_URL: str = f"{_BASE_URL}/languages"
    _DETECT_URL: str = f"{_BASE_URL}/detect"
    _LIMIT_SIZE = 102400

    def __init__(
        self,
        api_key: str,
        target: str = "zh-CN",
        source: str = None,
        fmt: str = "html",
        model: str = "nmt",
        proxies: Dict = None,
        timeout=None,
        trust_env=False,
    ):
        self.api_key = api_key
        self.target = target
        self.timeout = timeout
        self.source = source
        self.fmt = fmt
        self.model = model
        self.session = requests.Session()
        self.session.trust_env = trust_env
        if proxies:
            self.session.proxies = proxies

    def languages(
        self, target: str = None, model: str = None, timeout=...
    ) -> Union[List[LanguageResponse], Null]:
        """语言支持列表"""
        if target is None:
            target = self.target
        if model is None:
            model = self.model
        if timeout is ...:
            timeout = self.timeout
        response = self.session.get(
            self._LANGUAGE_URL,
            params={"key": self.api_key, "target": target, "model": model},
            timeout=timeout,
        )
        if response.status_code == 200:
            return [LanguageResponse(**i) for i in response.json()["data"]["languages"]]
        return Null(response)

    @overload
    def detect(self, q: str, timeout=...) -> DetectResponse:
        """..."""

    @overload
    def detect(self, q: List[str], timeout=...) -> List[DetectResponse]:
        """..."""

    def detect(
        self, q: Union[str, List[str]], timeout=...
    ) -> Union[DetectResponse, List[DetectResponse], Null]:
        """语言检测, 支持批量

        :param q: 字符串或字符串列表
        :param timeout: 超时时间， int | None
        :return: 成功则返回: :class:`pygtrans.TranslateResponse.DetectResponse` 对象,
            或 :class:`pygtrans.TranslateResponse.DetectResponse` 对象列表, 这取决于 `参数: q` 是字符串还是字符串列表.
            失败则返回 :class:`pygtrans.Null.Null` 对象

        基本用法:
            >>> from pygtrans import ApiKeyTranslate
            >>> client = ApiKeyTranslate(api_key='<api_key>')
            >>> d1 = client.detect('Hello')
            >>> d1.language
            'en'
            >>> assert isinstance(client.detect(['Hello', 'Google']), list)

        """
        if timeout is ...:
            timeout = self.timeout
        ll = []
        for ql in split_list(q):
            for qli in split_list_by_content_size(ql):
                for i in range(1, 4):
                    response = self.session.post(
                        self._DETECT_URL,
                        params={"key": self.api_key},
                        data={"q": qli},
                        timeout=timeout,
                    )
                    if response.status_code == 429:
                        time.sleep(5 * i)
                        continue
                    break
                # noinspection PyUnboundLocalVariable
                if response.status_code != 200:
                    return Null(response)
                ll.extend(
                    [
                        DetectResponse(**i[0])
                        for i in response.json()["data"]["detections"]
                    ]
                )
        if isinstance(q, str):
            return ll[0]
        return ll

    @overload
    def translate(
        self,
        q: str,
        target: str = None,
        source: str = None,
        fmt: str = None,
        model: str = None,
        timeout=...,
    ) -> TranslateResponse:
        """..."""

    @overload
    def translate(
        self,
        q: List[str],
        target: str = None,
        source: str = None,
        fmt: str = None,
        model: str = None,
        timeout=...,
    ) -> List[TranslateResponse]:
        """..."""

    def translate(
        self,
        q: Union[str, List[str]],
        target: str = None,
        source: str = None,
        fmt: str = None,
        model: str = None,
        timeout=...,
    ) -> Union[TranslateResponse, List[TranslateResponse], Null]:
        """文本翻译, 支持批量

        :param q: str: 字符串或字符串列表
        :param target: str: (可选)  目标语言, 默认: ``self.target``, :doc:`查看支持列表 <target>`
        :param source: str: (可选)  源语言, 默认: ``self.source``, :doc:`查看支持列表 <source>`
        :param fmt: str: (可选) 文本格式, ``text`` | ``html``, 默认: ``self.format``
        :param model: str: (可选) 翻译模型, ``nmt`` | ``pbmt``, 默认: ``self.model``
        :param timeout: 超时时间， int | None
        :return: 成功则返回: :class:`pygtrans.TranslateResponse.TranslateResponse` 对象,
            或 :class:`pygtrans.TranslateResponse.TranslateResponse` 对象列表, 这取决于 `参数: q` 是字符串还是字符串列表.
            失败则返回 :class:`pygtrans.Null.Null` 对象

        .. 谷歌API调用限制
            最大并发量: 128
            最大请求体大小: 102400 bytes

        基本用法:
            >>> from pygtrans import ApiKeyTranslate
            >>> client = ApiKeyTranslate(api_key='<api_key>')
            >>> text = client.translate('Google Translate')
            >>> text.translatedText
            '谷歌翻译'
            >>> text.detectedSourceLanguage
            'en'
            >>> texts = client.translate(['안녕하십니까', 'こんにちは'])
            >>> texts[0].translatedText, texts[1].translatedText
            ('你好', '你好')
        """

        if target is None:
            target = self.target
        if source == "auto":
            source = None
        if source is None:
            source = self.source
        if fmt is None:
            fmt = self.fmt
        if model is None:
            model = self.model
        if timeout is ...:
            timeout = self.timeout
        ll = []
        for ql in split_list(q):
            for qli in split_list_by_content_size(ql):
                for i in range(1, 4):
                    response = self.session.post(
                        self._BASE_URL,
                        params={
                            "key": self.api_key,
                            "target": target,
                            "source": source,
                            "format": fmt,
                            "model": model,
                        },
                        data={"q": qli},
                        timeout=timeout,
                    )
                    if response.status_code == 429:
                        time.sleep(5 * i)
                        continue
                    break
                # noinspection PyUnboundLocalVariable
                if response.status_code != 200:
                    return Null(response)

                ll.extend(
                    [
                        TranslateResponse(**i)
                        for i in response.json()["data"]["translations"]
                    ]
                )

        if isinstance(q, str):
            return ll[0]
        return ll



import requests


class Null:
    """
    :param response: 请求失败的 :class:`requests.Response` 对象
    """

    def __init__(self, response: requests.Response):
        self.response = response
        self.msg = f"{self.response.status_code}: {requests.status_codes._codes[self.response.status_code]}\n{self.response.text}"

    def __repr__(self):
        return self.msg



class LanguageResponse:
    """LanguageResponse"""

    def __init__(self, language: str, name: str = None):
        """

        :param language: 语言代码
        :param name: 语言名字
        """

        self.language = language
        self.name = name

    def __repr__(self):
        return (
            self.__class__.__qualname__
            + f"(language={repr(self.language)}, name={repr(self.language)})"
        )



"""谷歌翻译"""

from pygtrans.ApiKeyTranslate import ApiKeyTranslate
from pygtrans.DetectResponse import DetectResponse
from pygtrans.LanguageResponse import LanguageResponse
from pygtrans.Null import Null
from pygtrans.Translate import Translate
from pygtrans.TranslateResponse import TranslateResponse

__title__ = "pygtrans"
__description__ = "Google Translate, support APIKEY"
__url__ = "https://github.com/foyoux/pygtrans"
__version__ = '1.6.2'
__author__ = "foyoux"
__author_email__ = "yimi.0822@qq.com"
__license__ = "GPL-3.0"
__copyright__ = f"Copyright 2021-2024 {__author__}"
__ide__ = "PyCharm - https://www.jetbrains.com/pycharm/"

SOURCE_LANGUAGES = {
    "auto": "检测语言",
    "ab": "阿布哈兹语",
    "sq": "阿尔巴尼亚语",
    "aa": "阿法尔语",
    "ar": "阿拉伯语",
    "alz": "阿卢尔语",
    "am": "阿姆哈拉语",
    "ach": "阿乔利语",
    "as": "阿萨姆语",
    "az": "阿塞拜疆语",
    "awa": "阿瓦德语",
    "av": "阿瓦尔语",
    "ee": "埃维语",
    "ay": "艾马拉语",
    "ga": "爱尔兰语",
    "et": "爱沙尼亚语",
    "oc": "奥克语",
    "or": "奥利亚语",
    "om": "奥罗莫语",
    "os": "奥塞梯语",
    "tpi": "巴布亚皮钦语",
    "bew": "巴达维语",
    "ban": "巴厘语",
    "ba": "巴什基尔语",
    "eu": "巴斯克语",
    "btx": "巴塔克卡罗语",
    "bbc": "巴塔克托巴语",
    "bts": "巴塔克西马隆贡语",
    "bci": "巴乌雷语",
    "be": "白俄罗斯语",
    "bm": "班巴拉语",
    "pag": "邦阿西楠语",
    "pam": "邦板牙语",
    "bg": "保加利亚语",
    "nso": "北索托语",
    "bem": "奔巴语",
    "bik": "比科尔语",
    "bal": "俾路支语",
    "is": "冰岛语",
    "pl": "波兰语",
    "bs": "波斯尼亚语",
    "fa": "波斯语",
    "bho": "博杰普尔语",
    "bua": "布里亚特语",
    "br": "布列塔尼语",
    "bo": "藏语",
    "chm": "草原马里语",
    "ch": "查莫罗语",
    "ce": "车臣语",
    "chk": "楚克语",
    "cv": "楚瓦什语",
    "tn": "茨瓦纳语",
    "ts": "聪加语",
    "fa-AF": "达里语",
    "tt": "鞑靼语",
    "da": "丹麦语",
    "shn": "掸语",
    "tet": "德顿语",
    "de": "德语",
    "dv": "迪维希语",
    "dyu": "迪尤拉语",
    "tiv": "蒂夫语",
    "din": "丁卡语",
    "doi": "多格拉语",
    "ru": "俄语",
    "ndc-ZW": "恩道语",
    "nr": "恩德贝莱语（南部）",
    "dov": "恩敦贝语",
    "bm-Nkoo": "恩科字母（西非书面文字）",
    "fo": "法罗语",
    "fr": "法语",
    "sa": "梵语",
    "tl": "菲律宾语",
    "fj": "斐济语",
    "fi": "芬兰语",
    "fon": "丰语",
    "fy": "弗里西语",
    "fur": "弗留利语",
    "ff": "富拉尼语",
    "kg": "刚果语",
    "km": "高棉语",
    "kl": "格陵兰语",
    "ka": "格鲁吉亚语",
    "gom": "贡根语",
    "gu": "古吉拉特语",
    "gn": "瓜拉尼语",
    "cnh": "哈卡钦语",
    "kk": "哈萨克语",
    "ht": "海地克里奥尔语",
    "ko": "韩语",
    "ha": "豪萨语",
    "nl": "荷兰语",
    "hrx": "洪斯吕克语",
    "ky": "吉尔吉斯语",
    "ktu": "吉土巴语",
    "gl": "加利西亚语",
    "ca": "加泰罗尼亚语",
    "gaa": "加语",
    "cs": "捷克语",
    "kac": "景颇语",
    "kn": "卡纳达语",
    "kr": "卡努里语",
    "kha": "卡西语",
    "kek": "凯克其语",
    "kv": "科米语",
    "xh": "科萨语",
    "co": "科西嘉语",
    "crh": "克里米亚鞑靼语",
    "hr": "克罗地亚语",
    "qu": "克丘亚语",
    "ku": "库尔德语（库尔曼吉语）",
    "ckb": "库尔德语（索拉尼）",
    "trp": "廓克博若克语",
    "la": "拉丁语",
    "ltg": "拉特加莱语",
    "lv": "拉脱维亚语",
    "lo": "老挝语",
    "lt": "立陶宛语",
    "lij": "利古里亚语",
    "li": "林堡语",
    "ln": "林加拉语",
    "rn": "隆迪语",
    "luo": "卢奥语",
    "lg": "卢干达语",
    "lb": "卢森堡语",
    "rw": "卢旺达语",
    "lmo": "伦巴第语",
    "ro": "罗马尼亚语",
    "rom": "罗姆语",
    "mad": "马都拉语",
    "gv": "马恩岛语",
    "mg": "马尔加什语",
    "mwr": "马尔瓦迪语",
    "mt": "马耳他语",
    "mr": "马拉地语",
    "ml": "马拉雅拉姆语",
    "ms": "马来语",
    "ms-Arab": "马来语（爪夷文）",
    "mk": "马其顿语",
    "mh": "马绍尔语",
    "mam": "玛姆语",
    "mai": "迈蒂利语",
    "mfe": "毛里裘斯克里奥耳语",
    "mi": "毛利语",
    "mni-Mtei": "梅泰语（曼尼普尔语）",
    "mn": "蒙古语",
    "bn": "孟加拉语",
    "min": "米南语",
    "lus": "米佐语",
    "my": "缅甸语",
    "hmn": "苗语",
    "nhe": "纳瓦特尔语（东部瓦斯特卡）",
    "af": "南非荷兰语",
    "st": "南索托语",
    "ne": "尼泊尔语",
    "new": "尼泊尔语言（尼瓦尔语）",
    "nus": "努尔语",
    "no": "挪威语",
    "pap": "帕皮阿门托语",
    "pa": "旁遮普语（果鲁穆奇文）",
    "pa-Arab": "旁遮普语（沙木基文）",
    "pt": "葡萄牙语（巴西）",
    "pt-PT": "葡萄牙语（葡萄牙）",
    "ps": "普什图语",
    "ny": "齐切瓦语",
    "cgg": "奇加语",
    "ak": "契维语",
    "ja": "日语",
    "sv": "瑞典语",
    "zap": "萨巴特克语",
    "se": "萨米语（北部）",
    "sm": "萨摩亚语",
    "sr": "塞尔维亚语",
    "kri": "塞拉利昂克里奥尔语",
    "crs": "塞舌尔克里奥尔语",
    "sg": "桑戈语",
    "sat-Latn": "桑塔利语",
    "si": "僧伽罗语",
    "eo": "世界语",
    "sk": "斯洛伐克语",
    "sl": "斯洛文尼亚语",
    "ss": "斯瓦特语",
    "sw": "斯瓦希里语",
    "gd": "苏格兰盖尔语",
    "sus": "苏苏语",
    "ceb": "宿务语",
    "so": "索马里语",
    "tg": "塔吉克语",
    "ber": "塔马齐格特语（提非纳文）",
    "ber-Latn": "塔马塞特语",
    "ty": "塔希提语",
    "te": "泰卢固语",
    "ta": "泰米尔语",
    "th": "泰语",
    "to": "汤加语",
    "ti": "提格里尼亚语",
    "tcy": "图鲁语",
    "tum": "图姆布卡语",
    "tyv": "图瓦语",
    "tr": "土耳其语",
    "tk": "土库曼语",
    "war": "瓦瑞语",
    "mak": "望加锡语",
    "cy": "威尔士语",
    "vec": "威尼斯语",
    "ug": "维吾尔语",
    "ve": "文达语",
    "wo": "沃洛夫语",
    "udm": "乌德穆尔特语",
    "ur": "乌尔都语",
    "uk": "乌克兰语",
    "uz": "乌兹别克语",
    "es": "西班牙语",
    "szl": "西里西亚语",
    "scn": "西西里语",
    "iw": "希伯来语",
    "el": "希腊语",
    "hil": "希利盖农语",
    "haw": "夏威夷语",
    "sd": "信德语",
    "hu": "匈牙利语",
    "sn": "修纳语",
    "su": "巽他语",
    "jam": "牙买加土语",
    "sah": "雅库特语",
    "hy": "亚美尼亚语",
    "ace": "亚齐语",
    "iba": "伊班语",
    "ig": "伊博语",
    "ilo": "伊洛卡诺语",
    "it": "意大利语",
    "yi": "意第绪语",
    "hi": "印地语",
    "id": "印尼语",
    "en": "英语",
    "yua": "尤卡坦玛雅语",
    "yo": "约鲁巴语",
    "yue": "粤语",
    "vi": "越南语",
    "jw": "爪哇语",
    "zh-CN": "中文",
    "dz": "宗卡语",
    "zu": "祖鲁语",
}

TARGET_LANGUAGES = {
    "ab": "阿布哈兹语",
    "sq": "阿尔巴尼亚语",
    "aa": "阿法尔语",
    "ar": "阿拉伯语",
    "alz": "阿卢尔语",
    "am": "阿姆哈拉语",
    "ach": "阿乔利语",
    "as": "阿萨姆语",
    "az": "阿塞拜疆语",
    "awa": "阿瓦德语",
    "av": "阿瓦尔语",
    "ee": "埃维语",
    "ay": "艾马拉语",
    "ga": "爱尔兰语",
    "et": "爱沙尼亚语",
    "oc": "奥克语",
    "or": "奥利亚语",
    "om": "奥罗莫语",
    "os": "奥塞梯语",
    "tpi": "巴布亚皮钦语",
    "bew": "巴达维语",
    "ban": "巴厘语",
    "ba": "巴什基尔语",
    "eu": "巴斯克语",
    "btx": "巴塔克卡罗语",
    "bbc": "巴塔克托巴语",
    "bts": "巴塔克西马隆贡语",
    "bci": "巴乌雷语",
    "be": "白俄罗斯语",
    "bm": "班巴拉语",
    "pag": "邦阿西楠语",
    "pam": "邦板牙语",
    "bg": "保加利亚语",
    "nso": "北索托语",
    "bem": "奔巴语",
    "bik": "比科尔语",
    "bal": "俾路支语",
    "is": "冰岛语",
    "pl": "波兰语",
    "bs": "波斯尼亚语",
    "fa": "波斯语",
    "bho": "博杰普尔语",
    "bua": "布里亚特语",
    "br": "布列塔尼语",
    "bo": "藏语",
    "chm": "草原马里语",
    "ch": "查莫罗语",
    "ce": "车臣语",
    "chk": "楚克语",
    "cv": "楚瓦什语",
    "tn": "茨瓦纳语",
    "ts": "聪加语",
    "fa-AF": "达里语",
    "tt": "鞑靼语",
    "da": "丹麦语",
    "shn": "掸语",
    "tet": "德顿语",
    "de": "德语",
    "dv": "迪维希语",
    "dyu": "迪尤拉语",
    "tiv": "蒂夫语",
    "din": "丁卡语",
    "doi": "多格拉语",
    "ru": "俄语",
    "ndc-ZW": "恩道语",
    "nr": "恩德贝莱语（南部）",
    "dov": "恩敦贝语",
    "bm-Nkoo": "恩科字母（西非书面文字）",
    "fo": "法罗语",
    "fr": "法语",
    "sa": "梵语",
    "tl": "菲律宾语",
    "fj": "斐济语",
    "fi": "芬兰语",
    "fon": "丰语",
    "fy": "弗里西语",
    "fur": "弗留利语",
    "ff": "富拉尼语",
    "kg": "刚果语",
    "km": "高棉语",
    "kl": "格陵兰语",
    "ka": "格鲁吉亚语",
    "gom": "贡根语",
    "gu": "古吉拉特语",
    "gn": "瓜拉尼语",
    "cnh": "哈卡钦语",
    "kk": "哈萨克语",
    "ht": "海地克里奥尔语",
    "ko": "韩语",
    "ha": "豪萨语",
    "nl": "荷兰语",
    "hrx": "洪斯吕克语",
    "ky": "吉尔吉斯语",
    "ktu": "吉土巴语",
    "gl": "加利西亚语",
    "ca": "加泰罗尼亚语",
    "gaa": "加语",
    "cs": "捷克语",
    "kac": "景颇语",
    "kn": "卡纳达语",
    "kr": "卡努里语",
    "kha": "卡西语",
    "kek": "凯克其语",
    "kv": "科米语",
    "xh": "科萨语",
    "co": "科西嘉语",
    "crh": "克里米亚鞑靼语",
    "hr": "克罗地亚语",
    "qu": "克丘亚语",
    "ku": "库尔德语（库尔曼吉语）",
    "ckb": "库尔德语（索拉尼）",
    "trp": "廓克博若克语",
    "la": "拉丁语",
    "ltg": "拉特加莱语",
    "lv": "拉脱维亚语",
    "lo": "老挝语",
    "lt": "立陶宛语",
    "lij": "利古里亚语",
    "li": "林堡语",
    "ln": "林加拉语",
    "rn": "隆迪语",
    "luo": "卢奥语",
    "lg": "卢干达语",
    "lb": "卢森堡语",
    "rw": "卢旺达语",
    "lmo": "伦巴第语",
    "ro": "罗马尼亚语",
    "rom": "罗姆语",
    "mad": "马都拉语",
    "gv": "马恩岛语",
    "mg": "马尔加什语",
    "mwr": "马尔瓦迪语",
    "mt": "马耳他语",
    "mr": "马拉地语",
    "ml": "马拉雅拉姆语",
    "ms": "马来语",
    "ms-Arab": "马来语（爪夷文）",
    "mk": "马其顿语",
    "mh": "马绍尔语",
    "mam": "玛姆语",
    "mai": "迈蒂利语",
    "mfe": "毛里裘斯克里奥耳语",
    "mi": "毛利语",
    "mni-Mtei": "梅泰语（曼尼普尔语）",
    "mn": "蒙古语",
    "bn": "孟加拉语",
    "min": "米南语",
    "lus": "米佐语",
    "my": "缅甸语",
    "hmn": "苗语",
    "nhe": "纳瓦特尔语（东部瓦斯特卡）",
    "af": "南非荷兰语",
    "st": "南索托语",
    "ne": "尼泊尔语",
    "new": "尼泊尔语言（尼瓦尔语）",
    "nus": "努尔语",
    "no": "挪威语",
    "pap": "帕皮阿门托语",
    "pa": "旁遮普语（果鲁穆奇文）",
    "pa-Arab": "旁遮普语（沙木基文）",
    "pt": "葡萄牙语（巴西）",
    "pt-PT": "葡萄牙语（葡萄牙）",
    "ps": "普什图语",
    "ny": "齐切瓦语",
    "cgg": "奇加语",
    "ak": "契维语",
    "ja": "日语",
    "sv": "瑞典语",
    "zap": "萨巴特克语",
    "se": "萨米语（北部）",
    "sm": "萨摩亚语",
    "sr": "塞尔维亚语",
    "kri": "塞拉利昂克里奥尔语",
    "crs": "塞舌尔克里奥尔语",
    "sg": "桑戈语",
    "sat-Latn": "桑塔利语",
    "si": "僧伽罗语",
    "eo": "世界语",
    "sk": "斯洛伐克语",
    "sl": "斯洛文尼亚语",
    "ss": "斯瓦特语",
    "sw": "斯瓦希里语",
    "gd": "苏格兰盖尔语",
    "sus": "苏苏语",
    "ceb": "宿务语",
    "so": "索马里语",
    "tg": "塔吉克语",
    "ber": "塔马齐格特语（提非纳文）",
    "ber-Latn": "塔马塞特语",
    "ty": "塔希提语",
    "te": "泰卢固语",
    "ta": "泰米尔语",
    "th": "泰语",
    "to": "汤加语",
    "ti": "提格里尼亚语",
    "tcy": "图鲁语",
    "tum": "图姆布卡语",
    "tyv": "图瓦语",
    "tr": "土耳其语",
    "tk": "土库曼语",
    "war": "瓦瑞语",
    "mak": "望加锡语",
    "cy": "威尔士语",
    "vec": "威尼斯语",
    "ug": "维吾尔语",
    "ve": "文达语",
    "wo": "沃洛夫语",
    "udm": "乌德穆尔特语",
    "ur": "乌尔都语",
    "uk": "乌克兰语",
    "uz": "乌兹别克语",
    "es": "西班牙语",
    "szl": "西里西亚语",
    "scn": "西西里语",
    "iw": "希伯来语",
    "el": "希腊语",
    "hil": "希利盖农语",
    "haw": "夏威夷语",
    "sd": "信德语",
    "hu": "匈牙利语",
    "sn": "修纳语",
    "su": "巽他语",
    "jam": "牙买加土语",
    "sah": "雅库特语",
    "hy": "亚美尼亚语",
    "ace": "亚齐语",
    "iba": "伊班语",
    "ig": "伊博语",
    "ilo": "伊洛卡诺语",
    "it": "意大利语",
    "yi": "意第绪语",
    "hi": "印地语",
    "id": "印尼语",
    "en": "英语",
    "yua": "尤卡坦玛雅语",
    "yo": "约鲁巴语",
    "yue": "粤语",
    "vi": "越南语",
    "jw": "爪哇语",
    "zh-TW": "中文（繁体）",
    "zh-CN": "中文（简体）",
    "dz": "宗卡语",
    "zu": "祖鲁语",
}
